@@ -32,135 +32,22 @@ import tensorflow as tf
 from tensorflow.keras import backend, layers, models, optimizers
 
 from . import dataset, train, weights
+from ..common.base_class import BaseClass
 from ..model import yolov4
-from ..utility import media, predict
 
 
-class YOLOv4:
-    def __init__(self, tiny: bool = False):
+class YOLOv4(BaseClass):
+    def __init__(self, tiny: bool = False, tpu: bool = False):
         """
         Default configuration
         """
-        self.tiny = tiny
+        super(YOLOv4, self).__init__(tiny=tiny, tpu=tpu)
 
-        if tiny:
-            self.anchors = [
-                [[23, 27], [37, 58], [81, 82]],
-                [[81, 82], [135, 169], [344, 319]],
-            ]
-        else:
-            self.anchors = [
-                [[12, 16], [19, 36], [40, 28]],
-                [[36, 75], [76, 55], [72, 146]],
-                [[142, 110], [192, 243], [459, 401]],
-            ]
         self.batch_size = 32
         self.subdivision = 16
-        self._classes = None
         self._has_weights = False
         self.input_size = 608
         self.model = None
-        if tiny:
-            self.strides = [16, 32]
-        else:
-            self.strides = [8, 16, 32]
-        if tiny:
-            self.xyscales = [1.05, 1.05]
-        else:
-            self.xyscales = [1.2, 1.1, 1.05]
-
-    @property
-    def anchors(self):
-        """
-        Usage:
-            yolo.anchors = [12, 16, 19, 36, 40, 28, 36, 75,
-                            76, 55, 72, 146, 142, 110, 192, 243, 459, 401]
-            yolo.anchors = np.array([12, 16, 19, 36, 40, 28, 36, 75,
-                            76, 55, 72, 146, 142, 110, 192, 243, 459, 401])
-            print(yolo.anchors)
-        """
-        return self._anchors
-
-    @anchors.setter
-    def anchors(self, anchors: Union[list, tuple, np.ndarray]):
-        if isinstance(anchors, (list, tuple)):
-            self._anchors = np.array(anchors)
-        elif isinstance(anchors, np.ndarray):
-            self._anchors = anchors
-
-        if self.tiny:
-            self._anchors = self._anchors.astype(np.float32).reshape(2, 3, 2)
-        else:
-            self._anchors = self._anchors.astype(np.float32).reshape(3, 3, 2)
-
-    @property
-    def classes(self):
-        """
-        Usage:
-            yolo.classes = {0: 'person', 1: 'bicycle', 2: 'car', ...}
-            yolo.classes = "path/classes"
-            print(len(yolo.classes))
-        """
-        return self._classes
-
-    @classes.setter
-    def classes(self, data: Union[str, dict]):
-        if isinstance(data, str):
-            self._classes = media.read_classes_names(data)
-        elif isinstance(data, dict):
-            self._classes = data
-        else:
-            raise TypeError("YOLOv4: Set classes path or dictionary")
-
-    @property
-    def input_size(self):
-        """
-        Usage:
-            yolo.input_size = 608
-            print(yolo.input_size)
-        """
-        return self._input_size
-
-    @input_size.setter
-    def input_size(self, size: int):
-        if size % 32 == 0:
-            self._input_size = size
-        else:
-            raise ValueError("YOLOv4: Set input_size to multiples of 32")
-
-    @property
-    def strides(self):
-        """
-        Usage:
-            yolo.strides = [8, 16, 32]
-            yolo.strides = np.array([8, 16, 32])
-            print(yolo.strides)
-        """
-        return self._strides
-
-    @strides.setter
-    def strides(self, strides: Union[list, tuple, np.ndarray]):
-        if isinstance(strides, (list, tuple)):
-            self._strides = np.array(strides)
-        elif isinstance(strides, np.ndarray):
-            self._strides = strides
-
-    @property
-    def xyscales(self):
-        """
-        Usage:
-            yolo.xyscales = [1.2, 1.1, 1.05]
-            yolo.xyscales = np.array([1.2, 1.1, 1.05])
-            print(yolo.xyscales)
-        """
-        return self._xyscales
-
-    @xyscales.setter
-    def xyscales(self, xyscales: Union[list, tuple, np.ndarray]):
-        if isinstance(xyscales, (list, tuple)):
-            self._xyscales = np.array(xyscales)
-        elif isinstance(xyscales, np.ndarray):
-            self._xyscales = xyscales
 
     def make_model(
         self, activation0: str = "mish", activation1: str = "leaky",
@@ -176,6 +63,7 @@ class YOLOv4:
                 num_classes=len(self.classes),
                 xyscales=self.xyscales,
                 activation=activation1,
+                tpu=self.tpu,
             )
         else:
             self.model = yolov4.YOLOv4(
@@ -196,11 +84,17 @@ class YOLOv4:
         if weights_type == "yolo":
             weights.load_weights(self.model, weights_path, tiny=self.tiny)
         elif weights_type == "tf":
-            self.model.load_weights(weights_path).expect_partial()
+            self.model.load_weights(weights_path, by_name=True)
 
         self._has_weights = True
 
-    def save_as_tflite(self, tflite_path, quantization=None, data_set=None):
+    def save_as_tflite(
+        self,
+        tflite_path,
+        quantization=None,
+        data_set=None,
+        num_calibration_steps: int = 100,
+    ):
         """
         Save model and weights as tflite
 
@@ -212,7 +106,7 @@ class YOLOv4:
         converter = tf.lite.TFLiteConverter.from_keras_model(self.model)
 
         def representative_dataset_gen():
-            for _ in range(20):
+            for _ in range(num_calibration_steps):
                 # pylint: disable=stop-iteration-return
                 # TODO: # of iteration
                 images, _ = next(data_set)
@@ -241,51 +135,6 @@ class YOLOv4:
         with tf.io.gfile.GFile(tflite_path, "wb") as fd:
             fd.write(tflite_model)
 
-    def resize_image(self, image, ground_truth=None):
-        """
-        @param image:        Dim(height, width, channels)
-        @param ground_truth: [[center_x, center_y, w, h, class_id], ...]
-
-        @return resized_image or (resized_image, resized_ground_truth)
-
-        Usage:
-            image = yolo.resize_image(image)
-            image, ground_truth = yolo.resize_image(image, ground_truth)
-        """
-        return media.resize_image(
-            image, target_size=self.input_size, ground_truth=ground_truth
-        )
-
-    def candidates_to_pred_bboxes(self, candidates):
-        """
-        @param candidates: Dim(-1, (x, y, w, h, conf, prob_0, prob_1, ...))
-
-        @return Dim(-1, (x, y, w, h, class_id, probability))
-        """
-        return predict.candidates_to_pred_bboxes(candidates, self.input_size)
-
-    def fit_pred_bboxes_to_original(self, pred_bboxes, original_shape):
-        """
-        @param pred_bboxes:    Dim(-1, (x, y, w, h, class_id, probability))
-        @param original_shape: (height, width, channels)
-        """
-        # pylint: disable=no-self-use
-        return predict.fit_pred_bboxes_to_original(pred_bboxes, original_shape)
-
-    def draw_bboxes(self, image, bboxes):
-        """
-        @parma image:  Dim(height, width, channel)
-        @param bboxes: (candidates, 4) or (candidates, 5)
-                [[center_x, center_y, w, h, class_id], ...]
-                [[center_x, center_y, w, h, class_id, propability], ...]
-
-        @return drawn_image
-
-        Usage:
-            image = yolo.draw_bboxes(image, bboxes)
-        """
-        return media.draw_bboxes(image, bboxes, self.classes)
-
     #############
     # Inference #
     #############
