@@ -27,99 +27,82 @@ from typing import Union
 
 import cv2
 import numpy as np
-import tflite_runtime.interpreter as tflite
 
-from ..utility import media, predict
+try:
+    import tensorflow.lite as tflite
+except ModuleNotFoundError:
+    import tflite_runtime.interpreter as tflite
+
+from ..common import media, predict
+from ..common.base_class import BaseClass
 
 
-class YOLOv4:
-    def __init__(self):
+class YOLOv4(BaseClass):
+    def __init__(self, tiny: bool = False, tpu: bool = False):
         """
         Default configuration
         """
-        self._classes = None
+        super(YOLOv4, self).__init__(tiny=tiny, tpu=tpu)
+        self.grid_coord = []
         self.input_index = None
-        self.input_size = None
         self.interpreter = None
         self.output_index = None
-
-    @property
-    def classes(self):
-        """
-        Usage:
-            yolo.classes = {0: 'person', 1: 'bicycle', 2: 'car', ...}
-            yolo.classes = "path/classes"
-            print(len(yolo.classes))
-        """
-        return self._classes
-
-    @classes.setter
-    def classes(self, data: Union[str, dict]):
-        if isinstance(data, str):
-            self._classes = media.read_classes_names(data)
-        elif isinstance(data, dict):
-            self._classes = data
-        else:
-            raise TypeError("YOLOv4: Set classes path or dictionary")
+        self.output_size = None
 
     def load_tflite(self, tflite_path):
-        self.interpreter = tflite.Interpreter(model_path=tflite_path)
+        if self.tpu:
+            self.interpreter = tflite.Interpreter(
+                model_path=tflite_path,
+                experimental_delegates=[
+                    tflite.load_delegate("libedgetpu.so.1")
+                ],
+            )
+        else:
+            self.interpreter = tflite.Interpreter(model_path=tflite_path)
         self.interpreter.allocate_tensors()
         input_details = self.interpreter.get_input_details()[0]
         self.input_size = input_details["shape"][1]
         self.input_index = input_details["index"]
         output_details = self.interpreter.get_output_details()
-        self.output_index = [output_details[i]["index"] for i in range(3)]
-
-    def resize_image(self, image, ground_truth=None):
-        """
-        @param image:        Dim(height, width, channels)
-        @param ground_truth: [[center_x, center_y, w, h, class_id], ...]
-
-        @return resized_image or (resized_image, resized_ground_truth)
-
-        Usage:
-            image = yolo.resize_image(image)
-            image, ground_truth = yolo.resize_image(image, ground_truth)
-        """
-        return media.resize_image(
-            image, target_size=self.input_size, ground_truth=ground_truth
-        )
-
-    def candidates_to_pred_bboxes(self, candidates):
-        """
-        @param candidates: Dim(-1, (x, y, w, h, conf, prob_0, prob_1, ...))
-
-        @return Dim(-1, (x, y, w, h, class_id, probability))
-        """
-        return predict.candidates_to_pred_bboxes(candidates, self.input_size)
-
-    def fit_pred_bboxes_to_original(self, pred_bboxes, original_shape):
-        """
-        @param pred_bboxes:    Dim(-1, (x, y, w, h, class_id, probability))
-        @param original_shape: (height, width, channels)
-        """
-        # pylint: disable=no-self-use
-        return predict.fit_pred_bboxes_to_original(pred_bboxes, original_shape)
-
-    def draw_bboxes(self, image, bboxes):
-        """
-        @parma image:  Dim(height, width, channel)
-        @param bboxes: (candidates, 4) or (candidates, 5)
-                [[center_x, center_y, w, h, class_id], ...]
-                [[center_x, center_y, w, h, class_id, propability], ...]
-
-        @return drawn_image
-
-        Usage:
-            image = yolo.draw_bboxes(image, bboxes)
-        """
-        return media.draw_bboxes(image, bboxes, self.classes)
+        self.output_index = [details["index"] for details in output_details]
+        if self.tpu:
+            # sig, raw, sig, raw, ...
+            self.output_size = [
+                output_details[2 * i]["shape"][1]
+                for i in range(len(output_details) // 2)
+            ]
+            self.grid_coord = []
+            for _size in self.output_size:
+                xy_grid = np.meshgrid(np.arange(_size), np.arange(_size))
+                xy_grid = np.stack(xy_grid, axis=-1)
+                xy_grid = xy_grid[np.newaxis, ...]
+                self.grid_coord.append(xy_grid)
 
     #############
     # Inference #
     #############
 
+    def tpu_hair(self, x):
+        x = [np.split(_x, 3, axis=-1) for _x in x]
+
+        for i in range(len(self.output_size)):
+            for j in range(3):
+                # pylint: disable=unbalanced-tuple-unpacking
+                # sig
+                txty, _, conf_prob = np.split(x[2 * i][j], (2, 4), axis=-1)
+                # raw
+                _, twth, _ = np.split(x[2 * i + 1][j], (2, 4), axis=-1)
+                txty = (txty - 0.5) * self.xyscales[i] + 0.5
+                bxby = (txty + self.grid_coord[i]) / self.output_size[i]
+                bwbh = (self.anchors[i][j] / self.input_size) * np.exp(twth)
+                x[2 * i][j] = np.concatenate([bxby, bwbh, conf_prob], axis=-1)
+
+        pred = [
+            np.concatenate(x[2 * i], axis=-1)
+            for i in range(len(self.output_size))
+        ]
+        return pred
+
     def predict(self, frame: np.ndarray):
         """
         Predict one frame
@@ -138,8 +121,10 @@ class YOLOv4:
         self.interpreter.set_tensor(self.input_index, image_data)
         self.interpreter.invoke()
         candidates = [
-            self.interpreter.get_tensor(self.output_index[i]) for i in range(3)
+            self.interpreter.get_tensor(index) for index in self.output_index
         ]
+        if self.tpu:
+            candidates = self.tpu_hair(candidates)
         _candidates = []
         for candidate in candidates:
             grid_size = candidate.shape[1]
